/// <summary>
///  Nivel: el nivel actual de juego
///  @author Nacho Cabanes
///  @see Menu Juego
/// </summary>

/* --------------------------------------------------         
   Parte de FreddyTwo
 
   Versiones hasta la fecha:
   
   Num.   Fecha       Por / Cambios
   ---------------------------------------------------
   0.01  03-Feb-2012  Nacho Cabanes
                      Versin inicial: desglosado a partir de 
                        FreddyOne (Juego.cs)
   0.02  04-Feb-2012  Cristina Giner Olcina y Jorge Bernabeu Mira
                      Varios niveles consecutivos
 ---------------------------------------------------- */

using System;  // Para Random
using System.IO;  // Para fichero de niveles

public class Nivel
{
    // Atributos
    Random generador;

    //private int tiempoMax;
    private int premiosRestantes;
    private int premiosVisibles, enemigosVisibles, obstaculosVisibles;

    byte anchoFondo = 24;
    byte altoFondo = 17;
    short margenXFondo = 10;
    byte margenYFondo = 30;
    byte anchoCasillaFondo = 32;
    byte altoCasillaFondo = 32;
    Imagen imgPared;

    int numPremios, numEnemigos, numObstaculos;

    Obstaculo[] obstaculos;
    Enemigo[] enemigos;
    Premio[] premios;
    ElemGrafico[] fondos;
    int elementosFondo;
    int nivel;


    public byte[, ,] fondo;


    public Nivel()
    {
        leerDesdeFichero();

        numPremios = 20;
        numEnemigos = 12;
        numObstaculos = 20;
        nivel = 0;
        obstaculos = new Obstaculo[numObstaculos];
        enemigos = new Enemigo[numEnemigos];
        premios = new Premio[numPremios];
        generador = new Random();

        // y para cada uno de uno de ellos
        RellenarArrayDeFondo();

        // Preparo elementos
        for (int i = 0; i < numObstaculos; i++)  // Obstaculos
        {
            obstaculos[i] = new Obstaculo();
        }


        for (int i = 0; i < numEnemigos; i++)  // Enemigos
        {
            enemigos[i] = new Enemigo();
        }

        for (int i = 0; i < numPremios; i++)  // Premios
        {
            premios[i] = new Premio();
        }
        // Al empezar el primer nivel, al menos un precio debe estar visible
        premios[0].SetVisible(true);
        premiosRestantes = 1;
        imgPared = new Imagen("pared.png");


    }

    private void leerDesdeFichero()
    {
        StreamReader ficheroMapas = File.OpenText("niveles.dat");
        string linea;

        linea = ficheroMapas.ReadLine();
        int numeroNiveles = Convert.ToInt32(linea);

        fondo = new byte[numeroNiveles, altoFondo, anchoFondo];

        for (int nivelActual = 0; nivelActual < numeroNiveles; nivelActual++)
        {
            string separador = ficheroMapas.ReadLine();

            for (int lineaActual = 0; lineaActual < altoFondo; lineaActual++)
            {
                linea = ficheroMapas.ReadLine();
                for (int columnaActual = 0; columnaActual < anchoFondo; columnaActual++)
                    fondo [nivelActual, lineaActual, columnaActual] = 
                        Convert.ToByte(linea[columnaActual]-48);
                // Extraigo las columnas
            }
        }

        ficheroMapas.Close();
    }


    //funcion auxiliar
    public void RellenarArrayDeFondo()
    {

        elementosFondo = 0;
        for (int fila = 0; fila < altoFondo; fila++)  // Fondo
            for (int col = 0; col < anchoFondo; col++)
                if (fondo[nivel, fila, col] != 0)
                    elementosFondo++;
        // Y reservo espacio para el array que los contiene
        fondos = new ElemGrafico[elementosFondo];

        int posicFondo = 0;
        for (int fila = 0; fila < altoFondo; fila++)  // Fondo
            for (int col = 0; col < anchoFondo; col++)
                if (fondo[nivel, fila, col] != 0)
                {
                    fondos[posicFondo] = new ElemGrafico("pared.png", anchoCasillaFondo, altoCasillaFondo);
                    fondos[posicFondo].MoverA(margenXFondo + col * anchoCasillaFondo,
                        margenYFondo + fila * altoCasillaFondo);
                    fondos[posicFondo].SetVisible(true);
                    posicFondo++;
                }
    }


    /// Deja todo como es necesario para una nueva partida
    public void Reiniciar()
    {
        premiosVisibles = 1;
        enemigosVisibles = 0;
        obstaculosVisibles = 0;

        for (int i = 0; i < numObstaculos; i++)  // Obstaculos
        {
            obstaculos[i].MoverA(generador.Next(50, 700), generador.Next(30, 550));
            obstaculos[i].SetVisible(false);
        }


        for (int i = 0; i < numEnemigos; i++)  // Enemigos
        {
            enemigos[i].MoverA(generador.Next(50, 700), generador.Next(30, 550));
            enemigos[i].SetVisible(false);
        }

        for (int i = 0; i < numPremios; i++)  // Premios
        {
            do
            {
                premios[i].MoverA(generador.Next(50, 700),
                    generador.Next(30, 550));
                premios[i].SetVisible(true);
            }
            while (!EsPosibleMover(premios[i].GetX(), premios[i].GetY(),
                        premios[i].GetXFin(), premios[i].GetYFin()));
            premios[i].SetVisible(false);
        }
        // Al empezar el primer nivel, al menos un precio debe estar visible
        premios[0].SetVisible(true);
        premiosRestantes = 1;
    }


    public void DibujarOculta()
    {
        for (int i = 0; i < elementosFondo; i++)  // Imgenes del fondo
            fondos[i].DibujarOculta();

        for (int i = 0; i < numObstaculos; i++)  // Obstculos
            obstaculos[i].DibujarOculta();

        for (int i = 0; i < numEnemigos; i++)  // Enemigos
            enemigos[i].DibujarOculta();

        for (int i = 0; i < numPremios; i++)  // Premios
            premios[i].DibujarOculta();

    }

    // Devuelve si es posible mover a ciertas coordenadas
    // (no hay obstculos)
    public bool EsPosibleMover(int x, int y, int xFin, int yFin)
    {
        // Veo si choca con algn elemento del fondo
        for (int i = 0; i < elementosFondo; i++)
            if (fondos[i].ColisionCon(x,y,xFin,yFin))
                return false;

        // Si no ha chocado con ninguno, es posible moverse a esa posicin
        return true;
    }

    public int ObtenerPuntosPosicion(int x, int y, int xFin, int yFin)
    {
        // Veo si recoge algn premio
        for (int i = 0; i < numPremios; i++)
            if (premios[i].ColisionCon(x, y, xFin, yFin))
            {
                premios[i].SetVisible(false);
                premiosRestantes--;
                if (premiosRestantes == 0)
                    Avanzar();
                return 10;
            }

        // O si toca algn obstculo
        for (int i = 0; i < numObstaculos; i++)
            if (obstaculos[i].ColisionCon(x, y, xFin, yFin))
                return -1;

        // O si choca con algn enemigo
        for (int i = 0; i < numEnemigos; i++)
            if (enemigos[i].ColisionCon(x, y, xFin, yFin))
                return -1;

        // Si llego hasta aqui, es que no hay puntos que devolver
        return 0;

    }



    public int GetNumEnemigos()
    {
        return numEnemigos;
    }


    public Enemigo GetEnemigo(int num)
    {
        return enemigos[num];
    }


    public void Avanzar()
    {
        // Borro la pantalla y aviso del nuevo nivel
        Hardware.Pausa(200);
        Hardware.BorrarPantallaOculta();
        //Hardware.EscribirTextoOculta("Nivel: " + Convert.ToString(premiosVisibles + 1),
        //         350, 300, /* Coordenadas */ 255, 255, 255, /* Colores */ tipoDeLetra);
        Hardware.VisualizarOculta();
        Hardware.Pausa(1000);

        // Cambio los "ladrillos" del fondo
        if (nivel < 2)
            nivel++;
        else
            nivel = 0;

        RellenarArrayDeFondo();

        
        // Activo un enemigo mas
        if (enemigosVisibles < numEnemigos)
        {
            enemigosVisibles++;
            for (int i = 0; i < enemigosVisibles; i++)
                enemigos[i].SetVisible(true);
        }

        // Y un obstaculo mas
        if (obstaculosVisibles < numObstaculos)
        {
            obstaculosVisibles++;
            for (int i = 0; i < obstaculosVisibles; i++)
                obstaculos[i].SetVisible(true);
        }

        // Y un premio mas, y los recoloco
        if (premiosVisibles < numPremios)
        {
            premiosVisibles++;
            for (int i = 0; i < premiosVisibles; i++)
                do
                {
                    premios[i].MoverA(generador.Next(50, 700),
                        generador.Next(30, 550));
                    premios[i].SetVisible(true);
                }
                while (!EsPosibleMover(premios[i].GetX(), premios[i].GetY(),
                        premios[i].GetXFin(), premios[i].GetYFin()));
        }
        premiosRestantes = premiosVisibles;
    }

} /* Fin de la clase Nivel */

